/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */  

#include <config.h>
#include <nessuslib.h>
#include <nessus-devel.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <string.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <gtk/gtk.h>
#include <netdb.h>
#include "report.h"
#include "xstuff.h"
#include "families.h"
#include "xpm/black.xpm"
#include "xpm/red.xpm"
#include "report_file.h"
extern int show_pixmaps;
extern struct arglist * main_dialog;
struct report_data{
  GtkWidget *report_window;
  void *vdata;
  long ldata;
  struct arglist * hosts;
  GtkWidget * hosts_list;
  GtkWidget * ports_list;
  GtkWidget * holes_text;
  GtkWidget * hosts_win;
  GtkWidget * ports_win;
  GtkWidget * holes_win;
};


char * report_current_host;
extern struct arglist * hosts;

static void fill_hosts(struct arglist * , struct report_data * );
static void hosts_callback(GtkWidget * , struct report_data * );
static void ports_callback(GtkWidget * , struct report_data * );
static void update_ports(struct arglist * , char * , struct report_data * );
static void save_report_ask(GtkWidget *, GtkWidget *);
static void save_report(GtkWidget *, GtkWidget *);


/*
 * Creation of the reporting window
 */
static struct 
report_data *create_report_window(args)
  struct arglist * args;
{
  struct report_data * ret = emalloc(sizeof(struct report_data));
  GtkWidget * table;
  GtkWidget * label;
  GtkAdjustment * vadj;
  GtkWidget * vsb;
  GtkWidget * ttable;
  GtkWidget * button;
  ret->hosts = args;
  ret->report_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(ret->report_window), "destroy",
		     (GtkSignalFunc)close_window,ret->report_window);
  
  gtk_signal_connect(GTK_OBJECT(ret->report_window),"delete_event",
                    GTK_SIGNAL_FUNC(delete_event),NULL);
  
  gtk_window_set_title(GTK_WINDOW(ret->report_window), "Nessus Report");
  gtk_container_border_width(GTK_CONTAINER(ret->report_window), 10);     
  
  
  table = gtk_table_new(21,6,TRUE);
  gtk_table_set_row_spacing(GTK_TABLE(table),0,15);
  gtk_table_set_col_spacings(GTK_TABLE(table),5);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(ret->report_window), table);
  label = gtk_label_new("Hosts");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,2,0,1);
  gtk_widget_show(label);
  
  
  ret->hosts_win = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ret->hosts_win),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE(table), ret->hosts_win, 0,2,1,20);
  gtk_widget_show(ret->hosts_win);
  
  ret->hosts_list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(ret->hosts_win), ret->hosts_list);
  
  gtk_signal_connect(GTK_OBJECT(ret->hosts_list),
		     "selection_changed",
		     GTK_SIGNAL_FUNC(hosts_callback),
		     ret);
  gtk_widget_show(ret->hosts_list);
  
  
  label = gtk_label_new("Ports");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 2,3,0,1);
  gtk_widget_show(label);
  
  ret->ports_win = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ret->ports_win),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE(table), ret->ports_win, 2,3,1,20);
  gtk_widget_show(ret->ports_win);
  
  ret->ports_list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(ret->ports_win), ret->ports_list);
  gtk_signal_connect(GTK_OBJECT(ret->ports_list),
		     "selection_changed",
		     GTK_SIGNAL_FUNC(ports_callback),
		     ret);
  gtk_widget_show(ret->ports_list);
  
  label = gtk_label_new("Vulnerabilities");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 3,6,0,1);
  gtk_widget_show(label);
  
  vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
  vsb = gtk_vscrollbar_new (vadj);
  ret->holes_text = gtk_text_new(NULL,vadj);
  ttable = gtk_table_new(1,2,FALSE);
  ret->holes_win = ttable;
  
  gtk_table_attach_defaults(GTK_TABLE(table), ttable, 3,6,1,20);
  gtk_widget_show(ttable);
  gtk_table_set_col_spacing (GTK_TABLE (ttable), 0, 2);

  
  gtk_table_attach (GTK_TABLE (ttable), vsb, 1, 2, 0, 1,
                    0, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (ttable), ret->holes_text, 0, 1, 0, 1,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0); 
  button = gtk_button_new_with_label("Save...");
  gtk_table_attach(GTK_TABLE(table), button, 0,6,20,21,GTK_EXPAND | GTK_FILL,0,0,0);
  gtk_signal_connect(GTK_OBJECT(button),"clicked",
                    GTK_SIGNAL_FUNC(save_report_ask),NULL); 
                     
  gtk_widget_show(button);
  gtk_container_border_width (GTK_CONTAINER (ttable), 2);    
    gtk_widget_realize (ret->holes_text);
  gtk_text_set_editable(GTK_TEXT(ret->holes_text), FALSE);
  gtk_text_set_word_wrap(GTK_TEXT(ret->holes_text), TRUE);  
  gtk_widget_show(vsb);
  gtk_widget_show(ret->holes_text);
  gtk_widget_show(ret->report_window);
 
  return(ret);
}




/*
 * This function puts the names of the hosts tested in the first column (aka 
 * browser) of the window created by the function above
 */
static void 
fill_hosts(hosts, data)
 struct arglist * hosts;
 struct report_data * data;
{
  int vul=0;
  GList *dlist = NULL;
  
  while(hosts && hosts->next)
    {
      struct arglist * ports;
      
      ports = arg_get_value(hosts->value, "PORTS");
#ifdef DEBUG
      if(!ports)fprintf(stderr, "%s:%d : ports == NULL\n", __FILE__, __LINE__);
#endif
      while(ports && ports->next && vul!=2)
	{
	  struct arglist * reports;
	  struct arglist * infos;
	  
	  infos = arg_get_value(ports->value, "INFO");
	  reports = arg_get_value(ports->value, "REPORT");
	  if((int)(arg_get_value(ports->value, "STATE")) == 1)
	    {
	      if(infos)vul=1;
	      if(reports)vul=2;
	    }
	  ports = ports->next;
	}
      if(vul)
	{
	  GtkWidget * list_item;
	  GtkStyle *style;
	  GtkWidget * pixmapwid = NULL;
	  GdkPixmap * pixmap;
 	  GdkBitmap * mask;
 	  GtkWidget * box;
 	  GtkWidget * label;
          char * name;
 	  char ** pixdata = NULL;
 	  
	  if(show_pixmaps)
          {
	   if(vul==1)pixdata = blackdot_pixmap;
	   else pixdata = reddot_pixmap;
	   style = gtk_widget_get_style(data->ports_list);
	   pixmap = gdk_pixmap_create_from_xpm_d(data->ports_list->window, &mask,
            &style->bg[GTK_STATE_NORMAL],pixdata); 
           pixmapwid = gtk_pixmap_new(pixmap, mask);	
          }
          
	  list_item = gtk_list_item_new();
	  box = gtk_hbox_new(FALSE,10);
	  gtk_widget_show(box);
	  
	  if(show_pixmaps)
          {
          gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE,FALSE,0);
	  gtk_widget_show(pixmapwid);
          name = hosts->name;
	  }
          else
          {
          name = emalloc(strlen(hosts->name)+10);
          sprintf(name, "---%c %s", (vul==1)?'[':'!', hosts->name);
          }
          label = gtk_label_new(name);
	  gtk_box_pack_start(GTK_BOX(box), label, FALSE,FALSE,0);
	  gtk_widget_show(label);
	  
	  gtk_container_add(GTK_CONTAINER(list_item), box);
	  
          dlist = g_list_prepend(dlist, list_item);
          gtk_object_set_data(GTK_OBJECT(list_item),
			      "list_item_data",
			      hosts->name);
          gtk_widget_show(list_item);
	}
      else
        {
	  GtkWidget * list_item;
	  
	  list_item = gtk_list_item_new_with_label(hosts->name);
	  dlist = g_list_prepend(dlist, list_item);
	  gtk_object_set_data(GTK_OBJECT(list_item),
			      "list_item_data",
			      hosts->name);
	  gtk_widget_show(list_item);
	}
      hosts = hosts->next;
      vul = 0;
    }
  gtk_list_append_items(GTK_LIST(data->hosts_list), dlist);
}

/*
 * This function updates the portlist of the host which is currently clicked
 */
static void 
update_ports(hosts,cur_host,data)
     struct arglist * hosts;
     char * cur_host;
     struct report_data * data;
{
  struct arglist * ports;
  GList * dlist;
  /*
   * Seek for the host that interests us ...
   */
  while(hosts && hosts->next && strcmp(hosts->name, cur_host))
    hosts = hosts->next;
  
  dlist = GTK_LIST(data->ports_list)->children;
  if(dlist)gtk_list_remove_items(GTK_LIST(data->ports_list),dlist);
  
  
  gtk_text_set_point(GTK_TEXT(data->holes_text),0);
  gtk_text_forward_delete (GTK_TEXT(data->holes_text), 
  	gtk_text_get_length(GTK_TEXT(data->holes_text)));
  
  if(!hosts){
    fprintf(stderr, "Error ! Host selected not found !\n");
    return;
  }
  dlist = NULL;
  ports = arg_get_value(hosts->value, "PORTS");
  if(!ports)
    {
      /*
       * A given host MAY have NO port open, thus having a NULL
       * portlist is NOT an error
       */
      return;
    }
  else while(ports && ports->next)
    {
      struct arglist * reports;
      struct arglist * infos;
      
      
      reports = arg_get_value(ports->value, "REPORT");
      infos = arg_get_value(ports->value, "INFO");
      
      if((int)(arg_get_value(ports->value, "STATE")) == 1)
	{
	  /*
	   * If a vulnerability has been found on a port, then
	   * we write its number with a pixmap, so that the user sees
	   * immediatly what is interesting...
	   */
	  if(reports || infos)
	    {
	      GtkWidget * list_item;
	      GtkStyle *style;
	      GtkWidget * pixmapwid = NULL;
	      GdkPixmap * pixmap;
 	      GdkBitmap * mask;
 	      GtkWidget * box;
 	      GtkWidget * label;
              char * name;
 	      char ** pixdata = NULL;
 	  
              if(show_pixmaps)
              {
	      if(reports)pixdata = reddot_pixmap;
	      else pixdata = blackdot_pixmap;
	      
	      style = gtk_widget_get_style(data->ports_list);
	      pixmap = gdk_pixmap_create_from_xpm_d(data->ports_list->window, &mask,
                &style->bg[GTK_STATE_NORMAL],pixdata); 
              pixmapwid = gtk_pixmap_new(pixmap, mask);
	      }
              list_item = gtk_list_item_new();
	      box = gtk_hbox_new(FALSE,10);
	      gtk_widget_show(box);
    
              
              if(show_pixmaps)
              {
	      	gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE,FALSE,0);
	      	gtk_widget_show(pixmapwid);   
                name = emalloc(strlen(ports->name)+6);
                sprintf(name, "%s", ports->name);
	        
              }
              else
              {         
               name = emalloc(strlen(ports->name) + 10);
               sprintf(name, "---%c %s", reports ? '!':'[',ports->name); 
              } 
	      label = gtk_label_new(name);
	      gtk_box_pack_start(GTK_BOX(box), label, FALSE,FALSE,0);
	      gtk_widget_show(label);
	  
	      gtk_container_add(GTK_CONTAINER(list_item), box);
	      gtk_widget_show(list_item);
	      dlist = g_list_append(dlist, list_item);
	      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
				  ports->name);
	    }
	  else {
	    GtkWidget * list_item;
            char * n;
           
            n = emalloc(strlen(ports->name)+1);
            strncpy(n, ports->name, strlen(ports->name));
            
	    list_item = gtk_list_item_new_with_label(n);
	    dlist = g_list_append(dlist, list_item);
	    gtk_widget_show(list_item);
	    gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
				ports->name);
	  }
	}
      ports = ports->next;
    }
  if(dlist)gtk_list_append_items(GTK_LIST(data->ports_list), dlist);
}

/*
 * This function writes the holes found for a given port
 */
static void 
update_holes(hosts,cur_host,cur_port, data)
     struct arglist * hosts;
     char * cur_host;
     char * cur_port;
     struct report_data * data;
{
  struct arglist * ports;
  struct arglist * reports;
  struct arglist * infos;
  
  
  gtk_text_set_point(GTK_TEXT(data->holes_text),0);
  gtk_text_forward_delete (GTK_TEXT(data->holes_text), 
  	gtk_text_get_length(GTK_TEXT(data->holes_text)));
 
	
  /* 
   * Search for the host that interests us
   */
       while(hosts && hosts->next && 
             strcmp(hosts->name, cur_host))
         hosts = hosts->next;
       
       /*
        * Doh ! This should NOT happen
        */
       if(!hosts){
         fprintf(stderr, "Error ! Host selected not found !\n");
         return;
       }
       
       ports = arg_get_value(hosts->value, "PORTS");
       if(!ports){
         fprintf(stderr, "Error ! NULL portlist !\n");
         return;
       }
       
       while(ports && strcmp(ports->name, cur_port))
         ports = ports->next;
       
       if(!ports){
         fprintf(stderr, "Error ! Port selected not found !\n");
         return;
       }
       
      reports = arg_get_value(ports->value, "REPORT");
      			   
      while(reports && reports->next)
         {   			   
          gtk_text_insert(GTK_TEXT(data->holes_text), NULL,NULL,NULL, reports->value, -1);
          gtk_text_insert(GTK_TEXT(data->holes_text), NULL,NULL,NULL, "\n",-1);
          reports = reports->next;
         }
         
        infos = arg_get_value(ports->value, "INFO");
        while(infos && infos->next)
        {
          gtk_text_insert(GTK_TEXT(data->holes_text), NULL,NULL,NULL, infos->value, -1);
          gtk_text_insert(GTK_TEXT(data->holes_text), NULL,NULL,NULL, "\n",-1);
          infos = infos->next;
        }

}

static void 
hosts_callback(list, data)
 GtkWidget * list;
 struct report_data * data;
{
  char * cur_host = NULL;
  GList * dlist;
  GtkObject       *list_item;
  
  dlist = GTK_LIST(list)->selection;
  if(!dlist)return;
  list_item=GTK_OBJECT(dlist->data);  
  cur_host=gtk_object_get_data(list_item, "list_item_data");       
  update_ports(data->hosts, cur_host, data);
}

/*
 * This function is called whenever a port is being clicked
 * by the user
 */
static void 
ports_callback(list, data)
 GtkWidget * list; 
 struct report_data * data;
{
  char * cur_host = NULL;
  char * cur_port = NULL;
  GList * hlist,*plist;
  GtkObject * list_item;
  
  hlist = GTK_LIST(data->hosts_list)->selection;
  if(!hlist && !report_current_host){
  	fprintf(stderr, "I can't find the selected host!\n");
  	return;
  	}
  if(hlist)
  	{
  	list_item = GTK_OBJECT(hlist->data);
  	cur_host = gtk_object_get_data(list_item, "list_item_data");   
  	report_current_host = cur_host;
  	}
  else cur_host = report_current_host;
  
  
  plist = GTK_LIST(list)->selection;
  if(!plist)return;
  
  list_item = GTK_OBJECT(plist->data);
  cur_port = gtk_object_get_data(list_item, "list_item_data");
  
  update_holes(data->hosts, cur_host,cur_port, data);
}




/*
 *  Main function of the holes reporter
 *  Pretty clean, huh ? :)
 */
void 
report_tests(hosts)
 struct arglist * hosts;
{
  struct report_data * data;
  gtk_widget_show(arg_get_value(main_dialog, "WINDOW"));
  data = create_report_window(hosts);
  fill_hosts(hosts, data);
}


/*
 * Saves the report
 */
static void 
save_report_ask(nul,nul2)
  GtkWidget * nul;
  GtkWidget * nul2;
{
 GtkWidget * filew;
 
 if(!hosts)return;
 filew = gtk_file_selection_new ("File selection");
 gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                               "clicked", (GtkSignalFunc) save_report, filew );
           
 gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
           (filew)->cancel_button),
             "clicked", (GtkSignalFunc)close_window, (void *)filew);
 gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), "report.nsr");
           
 gtk_widget_show(filew);
}

/*
 * save_report
 *
 * this function is called when the user
 * clicks on the 'save' item of the file
 * menu...
 */
static void 
save_report(nul,filew)
    GtkWidget * nul;
    GtkWidget * filew;
{
 char * fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew));
 arglist_to_file(hosts, fname);
 gtk_widget_hide(filew);
 gtk_widget_destroy(filew);
}
 
