/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Attack.c : 
 *  launch the plugins, and manages the multithreading 
 *
 */
 
 
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h> 
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#ifdef HAVE_UCBINCLUDE
#include "/usr/ucbinclude/fcntl.h"
#else
#include <fcntl.h>
#endif /* HAVE_UCBINCLUDE */
#include <sys/ioctl.h>
#include <errno.h>

#include <config.h>
#include <nessuslib.h>
#include "attack.h"
#include "log.h"
#include "hostloop.h"
#include "sighand.h"
#include "rules.h"
#include "auth.h"
#include "threads.h"
#include "comm.h" 
#include "utils.h"
#include "nmap.h"
#include "preferences.h"
#include "ntp.h"
#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

/* 
 * Global variables 
 */
extern struct arglist * plugins;
extern struct arglist * preferences;
struct nessusd_threads * threads;
extern int global_socket;
pid_t current_plugin;
extern int ntp_version;
 
 
 
/*
 * Setup an arglist from a given hostname
 */
struct arglist * 
attack_init_hostinfos(hostname)
     char * hostname;
{
  struct arglist * hostinfos;
  struct arglist * ports;
  struct in_addr *p_addr;
  
  p_addr = emalloc(sizeof(struct in_addr));
  *p_addr = nn_resolve(hostname);
  if((p_addr->s_addr == INADDR_NONE) || (p_addr->s_addr == INADDR_ANY))
    {
      efree(&p_addr);
      return(NULL);
    }
  hostinfos = emalloc(sizeof(struct arglist));
  arg_add_value(hostinfos, "FQDN", ARG_STRING, strlen(hostname), hostname);
  arg_add_value(hostinfos, "IP", ARG_PTR, sizeof(struct in_addr), p_addr);
  ports = emalloc(sizeof(struct arglist));
  arg_add_value(hostinfos, "PORTS", ARG_ARGLIST, sizeof(struct arglist), ports);
  return(hostinfos);
}


/*
 * Add the informations about a host in a plugin arglist
 */
int 
attack_add_hostinfos(plugin, hostinfos)
     struct arglist *  plugin;
     struct arglist *  hostinfos;
{
  struct arglist * args;
  struct arglist * t;
  
  args = arg_get_value(plugin->value, "plugin_args");
  if(!args)
    {
#ifdef DEBUG
      log_write("error : a plugin has no \"plugin_args\" value !\n");
      log_write("dumping arglist\n");
      arg_dump(plugin->value, 0);
#endif
      return(-1);
    }
  t = arg_get_value(args, "HOSTNAME");
  if(t)arg_set_value(args, "HOSTNAME", sizeof(struct arglist), hostinfos);
  else arg_add_value(args, "HOSTNAME", ARG_ARGLIST, sizeof(struct arglist), hostinfos);
  return(0);
}

/*
 * This function attacks *one* host
 */
void 
attack_host_thread(hostinfos, plugins, hostname, port_range)
     struct arglist * hostinfos;
     struct arglist * plugins;
     char * hostname;
     char * port_range;
{ 
  int num_plugs = 0;
  int cur_plug = 1;
  
  num_plugs = get_active_plugins_number(plugins);
  
  /* do the portscan */
  nmap_main(hostinfos, port_range);
  
  /* launch the plugins */
  while(plugins && plugins->next)
    {
      plugin_run_t f_run;
      struct arglist * args;
      if(plug_get_launch(plugins->value)) /* can we launch it ? */
	{
	  if(attack_add_hostinfos(plugins, hostinfos))
	    continue;
	  f_run = (plugin_run_t)arg_get_value(plugins->value, "plugin_run");
	  args = arg_get_value(plugins->value, "plugin_args");
#ifdef LOGMORE
	  log_write("launching %s against %s\n", plugins->name,
		    hostname);
#endif
	  comm_send_status(hostname, "attack", cur_plug++, num_plugs);
	  if((current_plugin = fork())==0)
	    {
              /*
               * We fork before executing the plugin, 
               * so if it crashes of whatever, we are
               * still here
               */
               
	      /* call plugin_run() */
	      if(args)(*f_run)(args);
	      _exit(2);
	    }
	 else 
	   {
             /* 
              * Set up the timeout and wait...
              */
	     signal(SIGALRM, sighand_alarm_plugin);
	     alarm(PLUGIN_TIMEOUT);
	     waitpid(current_plugin, NULL, 0);
	     signal(SIGALRM, SIG_IGN);
	     alarm(0);
	   }
	}
      plugins = plugins->next;
    }
}

/* 
 * Start an attack : set up the host infos and launch the
 * attack thread
 */
void
attack_start(hostname, plugs, port_range)
     char * hostname;
     struct arglist * plugs;
     char * port_range;
{
 struct arglist * hostinfos;
 if(!(hostinfos = attack_init_hostinfos(hostname)))
   auth_printf("SERVER <|> ERROR <|> %s : host unknown <|> SERVER\n", 
	       hostname);
 else
   attack_host_thread(hostinfos, plugs, hostname, port_range);
 exit(0);
}

/*
 * This is the main function for attacking
 * a whole network
 */
int 
attack_host(preferences)
    struct arglist * preferences;
{
  int max_threads;
  struct arglist * tplugs;
  int num_tested = 0;
  int host_pending;
  int num_threads = 0;
  char * hostname;
  char * port_range;
  int max_hosts;
  int hl_flags;
  int subnet_class;
  hostname = arg_get_value(preferences, "TARGET");
  max_hosts = atoi(arg_get_value(preferences, "max_hosts"));
  port_range = arg_get_value(preferences, "port_range");
  /*
   * Initialization of the attack
   */
  
  set_socket_async(global_socket, sighand_io);
  tplugs = sort_plugins_by_type(plugins); /* sort the plugins by category   */
  if(ntp_version > NTP_10)send_plugin_order(tplugs);
  hl_flags = preferences_get_host_expansion(preferences);
  subnet_class = preferences_get_subnet_class(preferences);
  
  if(geteuid() && (hl_flags & HL_PING))
    {
     auth_printf("SERVER <|> ERROR <|> The server is not root. This is needed \
for sending a ping to the remote hosts <|> SERVER\n");
     comm_terminate();
     shutdown(global_socket, 2);
     exit(0);
     }
  hls_new_session(hostname, hl_flags, subnet_class);   /* init. the hostloop lib  */
  max_threads = get_max_thread_number(preferences);
  log_write("new attack against %s\n", hostname);
  threads = nessusd_threads_init();
  
  /*
   * Start the attack !
   */
  
  while(hostname && (num_tested != max_hosts))
    {
      pid_t pid;
      host_pending = 0 ;
      if(pTEST(get_hostname_attributes(hostname))) /* do we have the right to test this
						      host ? */
	{
	  if(num_threads < max_threads)
	    {
	      if((pid=fork())==0)
		attack_start(hostname, tplugs, port_range);
	      else { 
	  	nessusd_thread_register(&threads, hostname, pid);
                num_threads++;
	  	log_write("testing %s\n", hostname);
	      } 
	    }
	  else 
	    {
              /* 
               * There are too many threads at this time... We
               * have to wait until one of them finishes
               */
	      host_pending = 1; /* means that we must NOT update the hostname */
             
	      while(num_threads >= max_threads)
		num_threads = wait_for_a_thread(&threads);
	    }
	}
      if(!host_pending){
        num_tested++;
       	hostname = hls_get_next_host(); 
      }
    }
  /*
   * Every host is being tested... We have to wait for the threads
   * to terminate (maybe I could set up a timeout value here...)
   */
  
  while(wait_for_a_thread(&threads));
  
  /*
   * Clean up the plugin list in memory
   */
  while(tplugs)
  {
   struct arglist * next = tplugs->next;
   efree(&tplugs);
   tplugs = next;
  }
  return(0);
}

   
 
