/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Nessus Communication Manager -- it manages the NTP Protocol, version 1.0 and 1.1
 *
 */ 
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include "auth.h"
#include "comm.h" 
#include "threads.h"
#include "sighand.h"
#include "ntp.h"
#include "ntp_10.h"
#include "ntp_11.h"
#include "rules.h"
extern struct arglist * plugins;
extern int global_socket;
extern int ntp_version;
extern struct nessus_rules * rules;


/*
 * comm_init() :
 * Initializes the communication between the
 * server (us) and the client.
 */
void 
comm_init(soc)
     int soc;
{  
  char * buf = emalloc(20);
  
  /* We must read the version of the NTP the client
     wants us to use */
  signal(SIGALRM, sighand_alarm);
  alarm(CLIENT_TIMEOUT);   
  signal(SIGPIPE, sighand_pipe);      
  recv_line(soc, buf, 20);
  signal(SIGALRM, SIG_IGN);
  signal(SIGPIPE, SIG_IGN);
  alarm(0);              
  
  if(!strncmp(buf, "< NTP/1.0 >", 11))
    {
      ntp_version = NTP_10;
      send(soc, "< NTP/1.0 >\n",12,0);
    }
  else if(!strncmp(buf, "< NTP/1.1 >", 11))
    {
      ntp_version = NTP_11;
      send(soc, "< NTP/1.1 >\n", 12, 0);
    }  
  else
    {
      shutdown(soc, 2);
      exit(0);
    }
  efree(&buf);
}


/*
 * This function must be called at the end
 * of a session. 
 */
void 
comm_terminate()
{
  char * buf = emalloc(200);
  auth_printf("SERVER <|> BYE <|> BYE <|> SERVER\n");
  signal(SIGALRM, sighand_alarm);
  alarm(10);
  auth_gets(buf, 199); /* wait for the ack. of the client */
  efree(&buf);
  alarm(0);
}


/*
 * Sends the list of plugins that the server
 * could load to the client, using the 
 * NTP format
 */
void 
comm_send_pluginlist()
{
  struct arglist * l_plug = plugins;
  int i=1,j;
  char * categories[] = {"infos","attack","denial","unknown"};
  
  auth_printf("SERVER <|> PLUGIN_LIST <|>\n");
  while(l_plug && l_plug->next)
    {
      struct arglist * args;
      char * t;
      
      args = arg_get_value(l_plug->value, "plugin_args");
      t = arg_get_value(args, "DESCRIPTION");
      while((t=strchr(t,'\n')))t[0]=';';
      j = (int)arg_get_value(args, "CATEGORY");
      if((j>ACT_DENIAL) || (j<ACT_GATHER_INFO))j=4;
      auth_printf("%d <|> %s <|> %s <|> %s <|> %s <|> %s <|> %s\n", i, 
		  arg_get_value(args, "NAME"),
		  categories[j-1],
		  arg_get_value(args, "COPYRIGHT"),
		  arg_get_value(args, "DESCRIPTION"),
		  arg_get_value(args, "SUMMARY"),
		  plug_get_family(args));
      i++;
      l_plug = l_plug->next;
    }
  auth_printf("<|> SERVER\n");
}

/*
 * Sends the rules of the user
 */
void
comm_send_rules()
{
 struct nessus_rules * r = rules;
 if(ntp_version < NTP_11)return;
 auth_printf("SERVER <|> RULES <|>\n");
 while(r && r->next)
 {
  char c = 0;
  switch(r->category)
  { 
   case p_DONT_TEST :
    c = 'n';
    break;
   case p_DO_TEST :
    c = 'y';
    break;
   case p_DO_NOT_TEST :
    c = 'N';
    break;
   }
  if(c)auth_printf("%c:%s;\n", c, r->name);
  r = r->next;
 }
 auth_printf("<|> SERVER\n");
}

/*
 * Sends the preferences of the server
 */
void
comm_send_preferences(prefs)
 struct arglist * prefs;
{
 /* We have to be backward compatible with the NTP/1.0 */
 if(ntp_version < NTP_11)return;
 auth_printf("SERVER <|> PREFERENCES <|>\n");
 while(prefs && prefs->next)
 {
  auth_printf("%s <|> %s\n", 
  	prefs->name, prefs->value);
  prefs = prefs->next;
 }
 auth_printf("<|> SERVER\n");
}


/*
 * This function waits for the attack order
 * of the client
 * Meanwhile, it processes all the messages the client could
 * send
 */
void
comm_wait_order(preferences)
	struct arglist * preferences;
{
  char * str = emalloc(512);
  int problem = 1;
  
  while(problem)
  {
  auth_gets(str, 511);
  if(ntp_version < NTP_11)
   problem = ntp_10_parse_input(str, preferences);
  else
   problem = ntp_11_parse_input(str, preferences);
  }
}


/* 
 * Marks the plugins that will be
 * launched, according to the list
 * provided by the client (list of numbers
 * we should use)
 */
void 
comm_setup_plugins(id_list)
     char * id_list;
{
  char * list = id_list;
  char * buf;
  int id;
  int i = 0;
  int num_plugins=0;
  struct arglist * l_plugin;
  /* list = a string like '16;5;3' */
  
  l_plugin = plugins;
  while(l_plugin && l_plugin->next){
    num_plugins++;
    l_plugin = l_plugin->next;
  }
  buf = emalloc(strlen(id_list)+1);
  while(sscanf(list, "%d;%s", &id, buf) && strlen(list))
    {
      if(id==0)break;
      l_plugin = plugins;
      if(id<0)
 	{
	  struct arglist * lp = plugins;
	  while(lp && lp->next)
	    {
	      plug_set_launch(lp->value, 1);
	      lp = lp->next;
	    }
	  break;
 	}
      if(id>num_plugins)
	{
	  auth_printf("SERVER <|> ERROR <|> There's no plugin of id %d <|> SERVER\n", id);
	  l_plugin = NULL;
	}
      else for(i=1;i!=id;i++)l_plugin = l_plugin->next;
      if(l_plugin)plug_set_launch(l_plugin->value, 1);
      sprintf(list, "%s", buf);
      bzero(buf, strlen(id_list)+1);
    }
}


/*
 * Sends the status of an action
 */
void 
comm_send_status(hostname, action, current,max)
  char * hostname;
  char * action;
  int current, max;
{
  if(ntp_version == NTP_10)
  {
   auth_printf("SERVER <|> STAT <|> %s <|> %d/%d <|> SERVER\n",
	      hostname, current, max);
  }
  else auth_printf("SERVER <|> STATUS <|> %s <|> %s <|> %d/%d <|> SERVER\n",
  	      hostname, action, current, max);
}
