/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Nessus Communication Manager -- it manages the NTP Protocol, version 1.1
 *
 */ 
 
#include <config.h>
#include <nessuslib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include "ntp.h"
#include "parser.h"
#include "ntp_11.h"
#include "comm.h"
#include "auth.h"
#include "rules.h"
#define DEBUGMORE

static int ntp_11_prefs(struct arglist *);
static int ntp_11_read_prefs(struct arglist *);
static void ntp_11_send_prefs_errors(struct arglist *);
static int ntp_11_rules();
static int ntp_11_new_attack(char *, struct arglist *);
/*
 * Parses the input sent by the client before
 * the NEW_ATTACK message.
 */
int ntp_11_parse_input(input, preferences)
   char * input;
   struct arglist * preferences;
{
 char * str;
 char * orig = emalloc(strlen(input)+1);
 strncpy(orig, input, strlen(input));
 
 str = strchr(input, '<');
 str = str - 1;
 str[0] = 0;
 if(!strcmp(input, "CLIENT"))
 {
  input = str + 5;
  str = strchr(input, ' ');
  if(str)str[0]=0;
  if(input[strlen(input)-1]=='\n')input[strlen(input)-1]=0;
  if(!strcmp(input, "PREFERENCES")){
  	ntp_11_prefs(preferences);
        return(1);
        }
  if(!strcmp(input, "RULES")){
  	ntp_11_rules();
        return(1);
        }
  if(!strcmp(input, "NEW_ATTACK"))return(ntp_11_new_attack(orig, preferences));
  return(1);
 }
 return(1);
}



static int ntp_11_prefs(preferences)
 struct arglist * preferences;
{
 int problem;
 
 problem = ntp_11_read_prefs(preferences);
 if(!problem)ntp_11_send_prefs_errors(preferences);
 return(problem);
}



static int ntp_11_read_prefs(preferences)
 struct arglist * preferences;
{
 int finished = 0;
 char * input = emalloc(1024);
 while(!finished)
 {
  auth_gets(input, 1023);
  if(strstr(input, "<|> CLIENT"))finished = 1;
  else
  {
   char * pref;
   char * value;
   char * v;
 
    pref = input;
    v = strchr(input, '<');
    v-=1;
    v[0] = 0;
    
    value = v + 5;
    v = emalloc(strlen(value));
    strncpy(v, value, strlen(value)-1);
   if(arg_get_value(preferences, pref))
    arg_set_value(preferences, pref, strlen(v), v);
   else
    arg_add_value(preferences, pref, ARG_STRING, strlen(v), v);
  }
 }
 efree(&input);
 return(0);
}


static void ntp_11_send_prefs_errors(preferences)
 struct arglist * preferences;
{
 /* not implemented yet */
 auth_printf("SERVER <|> PREFERENCES_ERRORS <|>\n");
 auth_printf("<|> SERVER\n");
}



static int ntp_11_rules()
{
 char * main_rules = emalloc(10);
 char * buffer;
 int finished = 0;
 
 buffer = emalloc(1024); 
 while(!finished)
 {
  auth_gets(buffer, 1023);
  if(strstr(buffer, "<|> CLIENT"))finished = 1;
  else
   {
    int len;
    char * t, *k;
    buffer[strlen(buffer)-1]=0;
    len = strlen(buffer) + strlen(main_rules) + 1;
    t = emalloc(len);
    sprintf(t, "%s%s", main_rules, buffer);
    k = main_rules;
    main_rules = t;
    efree(&k);
    }
  }
 efree(&buffer);
 buffer = main_rules;
 while((buffer=strchr(buffer, ';')))buffer[0]='\n';
 rules_add(main_rules, 2);
 return(0);
}



static int ntp_11_new_attack(input, preferences)
	char * input;
	struct arglist * preferences;
{
 char * target = emalloc(strlen(input));
 char * clean_target;
 char * plugin_set;
 
 sscanf(input, "CLIENT <|> NEW_ATTACK <|> %s <|> CLIENT\n", target);
 if(!strlen(target)){
  efree(&target);
  return(1);
 }
 clean_target = emalloc(strlen(target)+1);
 strncpy(clean_target, target, strlen(target));
 
 plugin_set = arg_get_value(preferences, "plugin_set");
 comm_setup_plugins(plugin_set);
 if(arg_get_value(preferences, "TARGET"))
  arg_set_value(preferences, "TARGET", strlen(clean_target), clean_target);
 else
  arg_add_value(preferences, "TARGET", ARG_STRING, strlen(clean_target), clean_target);
 efree(&target);
 return(0);
}
 
