/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Nessus Communication Manager -- it manages the NTP Protocol, version 1.0
 *
 */ 
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>  
#include "auth.h"
#include "threads.h"


/*
 * Initializes the global table of threads (GTOT)
 */
struct nessusd_threads * nessusd_threads_init()
{
 struct nessusd_threads * threads  = emalloc(sizeof(struct nessusd_threads));
 return(threads);
}

/*
 * Register a thread in the GTOT
 */
void nessusd_thread_register(threads, hostname, pid)
 struct nessusd_threads ** threads;
 char * hostname;
 pid_t pid;
{  
  struct nessusd_threads * t = *threads;
  
  while(t && t->next)t = t->next;
  if(!t)return;
  t->hostname = emalloc(strlen(hostname)+1);
  strncpy(t->hostname, hostname, strlen(hostname));
  t->pid = pid;
  t->next = emalloc(sizeof(struct nessusd_threads));
  t->next->previous = t;
}

/*
 * Write in the GTOT that a thread is finished.
 */
void nessusd_thread_unregister(threads, thread)
   struct nessusd_threads ** threads;
   struct nessusd_threads * thread;
{
  if(thread->previous)thread->previous->next = thread->next;
  else *threads = thread->next;
  if(thread->next)thread->next->previous = thread->previous;
  efree(&thread->hostname);
  efree(&thread);
}

/* 
 * Kill all the threads that are in charge of the 
 * host <hostname>
 */
void nessusd_thread_kill_by_name(threads, thread, hostname)
     struct nessusd_threads ** threads;
     struct nessusd_threads * thread;
     char * hostname;
{
  struct nessusd_threads * t = thread;
  while(t && t->next && t->hostname && strcmp(hostname, t->hostname))t = t->next;
  if(t && t->hostname && !strcmp(hostname, t->hostname))
    {
      struct nessusd_threads * next = t->next;
      kill(t->pid, SIGKILL);
      nessusd_thread_unregister(threads, t);
      nessusd_thread_kill_by_name(threads, next, hostname); 
    }
}


/*
 * Kill all the threads
 */
void nessusd_thread_kill_all(threads)
     struct nessusd_threads * threads;
{
 struct nessusd_threads * t = threads;
 
 while(t && t->next)
 {
  kill(t->pid, SIGKILL);
  nessusd_thread_unregister(&threads, t);
  t = t->next;
 }
}
  
   


/*
 * Returns the number of threads, and waits for
 * those which are not finished yet.
 */
int
wait_for_a_thread(threads)
  struct nessusd_threads ** threads;
{
  struct nessusd_threads * t = *threads;
  int num_threads = 0;
  
  while(t && t->next)
    {
     struct nessusd_threads * next = t->next;
     int status = 0;
     int code = waitpid(t->pid, &status, WNOHANG);
     
     num_threads++;
     if(code==-1 || (code>0 && WIFEXITED(status)))
	{ 
	      nessusd_thread_unregister(threads, t);
	      num_threads--;
	}
      t = next;
    }
  return(num_threads);
}
   
   

