/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
 
#include <config.h>
#include <nessuslib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include <sys/types.h>
#include <sys/wait.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h> 
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#ifdef HAVE_UCBINCLUDE
#include "/usr/ucbinclude/fcntl.h"
#else
#include <fcntl.h>
#endif /* HAVE_UCBINCLUDE */
#include <sys/ioctl.h>
#include <errno.h>


#include "log.h"
#include "ntp.h"
#include "auth.h"

extern int ntp_version;

/*
 * Sort the plugins by type (GATHER_INFO, ATTACK and DENIAL)
 */
struct arglist * sort_plugins_by_type(lplugs)
 struct arglist * lplugs;
{
 struct arglist * plugins = lplugs;
 struct arglist * ret;
 struct arglist * tplugs = emalloc(sizeof(struct arglist));
 int i;
 ret = tplugs;
 for(i=ACT_GATHER_INFO;i<=ACT_DENIAL;i++)
    {
      while(lplugs && lplugs->next)
	{
	  struct arglist * args;
	  
	  args = arg_get_value(lplugs->value, "plugin_args");
	  if(args && ((int)arg_get_value(args, "CATEGORY")==i))
	    {
	      tplugs->value = lplugs->value;
	      tplugs->name = lplugs->name;
	      tplugs->type = lplugs->type;
	      tplugs->length = lplugs->length;
	      tplugs->next = emalloc(sizeof(struct arglist));
	      tplugs = tplugs->next;
	    }
	  lplugs = lplugs->next;
	}
      lplugs = plugins;
    }
   return(ret);
}

/* 
 * Get the max number of threads that
 * the user allowed us to use. If it's not
 * set correctly, we return 5
 */
int get_max_thread_number(preferences)
 struct arglist * preferences;
{
  int max_threads;
  if(arg_get_value(preferences, "max_threads"))
    {
      max_threads = atoi(arg_get_value(preferences, "max_threads"));
      if(max_threads<=0)
	{
	  log_write("Error ! max_threads = %d -- check "NESSUSD_CONF"\n", 
		    max_threads);
	  max_threads = 1;
	}
    }
  else max_threads = 5;
  return(max_threads);
}


/*
 * Returns the number of plugins that will be launched
 */
int 
get_active_plugins_number(plugins)
 struct arglist *  plugins;
{
  int i = 0;
  while(plugins && plugins->next)
    { 
      if(plug_get_launch(plugins->value))i++;
      plugins = plugins->next;
    }
 return(i);
}


/* 
 * Make a socket async
 */
void
set_socket_async(socket, callback)
 int socket;
 void * callback;
{
  signal(SIGIO, callback);
  fcntl(socket, F_SETOWN, getpid());
  fcntl(socket, F_SETFL, FASYNC); 
}



void send_plugin_order(plugins)
 struct arglist * plugins;
{
 int num; 
 char * str;
 
 if(ntp_version > NTP_10)
 {
  num = get_active_plugins_number(plugins);
  str = emalloc(num*4);
  while(plugins && plugins->next)
  {
    char * sp;
    
    if(plug_get_launch(plugins->value))
     {
      struct arglist *v = arg_get_value(plugins->value, "plugin_args");
      sp = emalloc(9);
      sprintf(sp, "%d", (int)arg_get_value(v, "ID"));
      strcat(str, sp);
      efree(&sp);
      strcat(str, ";");
     }
     plugins = plugins->next;
   }
  auth_printf("SERVER <|> PLUGINS_ORDER <|> %s <|> SERVER\n",
  		str);
  efree(&str);
 }
}


void plugins_set_ntp_version(plugins, version)
 struct arglist * plugins;
 int version;
{
 while(plugins && plugins->next)
 {
  struct arglist * v;
  
  v = arg_get_value(plugins->value, "plugin_args");
  if(v)arg_add_value(v, "NTP_VERSION", ARG_INT, sizeof(int), (void *)version);
  plugins = plugins->next;
 }
}
